<?php
namespace HCM\FPA\Controllers;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\FPA\Services\ItemLoteLancService;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use \Folha\Entities\FpaItecalcfolha;
use \Folha\Entities\FpaCalculofolha;
use \Folha\Entities\FpaItemlotelanc;
use \Folha\Entities\FpaLotelancevento;
use \Folha\Entities\PtoResumositufreq;
use HCM\Util\HCMParameters;
use HCM\Util\MsgCalculo;
use HCM\Util\Messages;
use HCM\Util\FPA;

class ItemLoteLancController extends Crud {

    protected $dataSourceGetItemLoteLanc = 'getItemLoteLanc';
protected $dataSourceSetItemLoteLanc = 'fpaItemLoteLanc_setItemLoteLanc';
    protected $dataSourceSetIteCalcFolha = 'fpaIteCalcFolha_setIteCalcFolha';

    const APROVADO = 'APROVADO';
    const REPROVADO = 'REPROVADO';

    protected $parameterBagColumns = array('NRORG', 'DTMESCOMPETENC', 'NRORGPADRAO', 'NRLOTELANCEVENTO');
    protected $itemLoteLancService;

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, EntityManager $entityManager, HCMParameters $hcmParameters, ItemLoteLancService $itemLoteLancService){
        $this->parameterBag    = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->itemLoteLancService = $itemLoteLancService;
        $this->entityManager = $entityManager;
        parent::__construct($dataSourceManager);
    }

    public function getItemLoteLanc (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            set_time_limit(0);
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceGetItemLoteLanc);
            $newFilterData = $this->populateParameterBag($filterData);
            $newFilterData->setPageSize(99999999999);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setItemLoteLanc (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetItemLoteLanc;
        try{
            $msg = array();
            $disponibilidade = true;
            foreach($request->getDataSet()->getRows() as $row) {
                if (isset($row['NRTIPOVINCULOM'])) {
                    $bloq = $this->itemLoteLancService->verificaEventoBloqueado($row['NRORG'], $row['NREVENTOM'], $row['NRTIPOVINCULOM']);
                    if ($bloq !== null){
                        if ($bloq['NREVENTOMPARA'] === null) {
                            $this->msg[] = array('msg' => 'Operação bloqueada. O evento '.$row['NREVENTOM'].' não pode ser lançado para o vínculo "'.$row['NRVINCULOM'].' - '.$row['NMVINCULOM']. '"!');
                            $disponibilidade = false;
                        } else {
                            $msg[] = array('msg' => 'Operação bloqueada.  O evento '.$row['NREVENTOM'].' não pode ser lançado para o vínculo "'.$row['NRVINCULOM'].' - '.$row['NMVINCULOM'].'" e foi ajustado para o evento '.$bloq['NREVENTOMPARA'].'!');
                            $disponibilidade = true;
                            $row['NREVENTOM'] = $bloq['NREVENTOMPARA'];
                        }
                    } else {
                        $disponibilidade = true;
                    }
                }
                
                if ($disponibilidade === true) {
                    if($row['__is_new'] === true){
                        $row['DTINCLUSAO'] = $this->hcmParameters->getDataAtual(false);
                        $row['CDOPERINCLUSAO'] = $row['CDOPERADOR_LOG'];
                        $row['NRORGINCLUSAO'] = $row['NRORGTRAB'];
                        $row['DTULTATU'] = $this->hcmParameters->getDataAtual(false);
                        $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                        $row['NRORGULTATU'] = $row['NRORGTRAB'];
                        $row['NRITLOTELANC'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORGTRAB'], 'FPA_ITEMLOTELANC', 1, 12, true);
                        
                        $itensLote = \Setforms2\Shared\Registry::getEntityManager()->getRepository(Repositories::FPA_ITEMLOTELANC)->findBy(array('nrorg' => $this->hcmParameters->getNrorgTrabalho(),
                        'nrlotelancevento' => $row['NRLOTELANCEVENTO'], 'nreventom' => $row['NREVENTOM'], 'nrvinculom' => $row['NRVINCULOM']));  
                        
                        if($itensLote){
                            throw new \Exception('Evento já cadastrado para o vínculo informado!', 10);
                        }
                        
                    } else {
                        $row['DTULTATU'] = $this->hcmParameters->getDataAtual(false);
                        $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                        $row['NRORGULTATU'] = $row['NRORGTRAB'];
                    }
                    
                    if(isset($row['VREVENTOLOTE'])){
                        $row['VREVENTOLOTE'] = str_replace('.', '', $row['VREVENTOLOTE']);
                        $row['VREVENTOLOTE'] = str_replace(',', '.', $row['VREVENTOLOTE']);
                        $row['VREVENTOLOTE'] = abs($row['VREVENTOLOTE']);
                    }
                    
                    if(!isset($row['VRREFFOLHALOTE'])){
                        $row['VRREFFOLHALOTE'] = 0;
                    }
                    
                    $row['NRORG'] = $row['NRORGTRAB'];
                    
                    parent::save($request, $response);
                }
            }
            if (count($msg) > 0) {
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet("msg", $msg));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setIteLoteInline (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            error_reporting(E_ERROR | E_PARSE);
            foreach ($request->getDataSet()->getRows() as &$rows) {
                foreach ($rows as &$row) {    
                    if(array_key_exists('VRREFFOLHA_inline', $row)){
                        $row['__is_new'] = false;
                    }
                
                    $this->itemLoteLancService->updateIteLoteInline($row);
                    
                    $message = new \Zeedhi\Framework\DTO\Response\Message('Registro salvo com sucesso', DTO\Response\Message::TYPE_MESSAGE);
                    $response->addMessage($message);
                }
            }
            
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteItemLoteLanc (DTO\Request\Row $row, DTO\Response $response) {
        try{
            $dataSet = new DataSet($this->dataSourceSetItemLoteLanc, array($row->getRow()));
            $deletedRows = $this->dataSourceManager->delete($dataSet);

            $response->addDataSet(new DataSet($this->dataSourceSetItemLoteLanc, array('type' => 'success', $deletedRows)));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     public function deleteItensLoteChecked (DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0);
        try {
            $dataset = $request->getDataSet()->getRows();
            $row = $dataset[0];
            
            $this->itemLoteLancService->deleteItensLoteChecked($row);    
            
            $message = new \Zeedhi\Framework\DTO\Response\Message('Delete dos Itens finalizado com sucesso', DTO\Response\Message::TYPE_MESSAGE);
            if($message instanceof \Zeedhi\Framework\DTO\Response\Message){
                $response->addMessage($message);
            }
        
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }   
    }
    

    /*
    public function cancelarLote (DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->itemLoteLancService->cancelarLote($request->getDataSet()->getRows()[0]['NRORGTRAB'], $request->getDataSet()->getRows()[0]['NRLOTELANCEVENTO']);
            // $this->itemLoteLancService->updateIdStatusLoteLanc(self::REPROVADO, $request->getDataSet()->getRows()[0]['NRLOTELANCEVENTO'], $request->getDataSet()->getRows()[0]['NRORGTRAB']);
            $response->addDataSet(new DataSet('cancelarLote', array('type' => 'success')));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    */
    
    public function cancelarLote (DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach($request->getDataSet()->getRows() as $row) {
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                $temLiquido = false;
                $vincLiquido = array();
                $msgs = array();
                $calculofolha = $nrcalculomsg = null;
                $dsorigmsg = 'LOTEEVENTO'.$row['NRLOTELANCEVENTO'];
                
                $params = [
                            'P_NRORG'            => $nrorg,
                            'P_DTOCORRENCIA'     => $this->hcmParameters->getDtCompetencia(),
                            'P_NRTIPOMOVIMENT'   => $row['NRTIPOMOVIMENT'],
                            'P_NRLOTELANCEVENTO' => $row['NRLOTELANCEVENTO'],
                          ];
                
                $where = "NRORG = :P_NRORG 
                          AND DTOCORRENCIA = :P_DTOCORRENCIA 
                          AND NRTIPOMOVIMENT = :P_NRTIPOMOVIMENT 
                          AND NRCALCULOFOLHA IN (
                                                    SELECT NRCALCULOFOLHA 
                                                    FROM FPA_ITECALCFOLHA 
                                                    WHERE NRORG = :P_NRORG 
                                                    AND NRLOTELANCEVENTO = :P_NRLOTELANCEVENTO
                                                )
                ";
                
                $fpacalculofolha = FpaCalculofolha::getObjectsByWhere($where, $params);
    
                if($fpacalculofolha){
                    $calculofolha = $fpacalculofolha[0];    
                    $nrcalculomsg = $calculofolha->getNrcalculofolha();
                } 

                $params = [
                            'P_NRORG'            => $nrorg,
                            'P_DTCOMPETLOTEEVE'  => $this->hcmParameters->getDtCompetencia(),
                            'P_NRLOTELANCEVENTO' => $row['NRLOTELANCEVENTO'],
                          ];
                          
                $where = 'NRORG = :P_NRORG 
                          AND NRLOTELANCEVENTO IN (
                                                    SELECT NRLOTELANCEVENTO 
                                                    FROM FPA_LOTELANCEVENTO 
                                                    WHERE NRORG = :P_NRORG 
                                                    AND NRLOTELANCEVENTO = :P_NRLOTELANCEVENTO 
                                                    AND FPA_LOTELANCEVENTO.DTCOMPETLOTEEVE = :P_DTCOMPETLOTEEVE
                                                   )
                ';
                
                $itenslanclote = FpaItemlotelanc::getObjectsByWhere($where, $params);
    
                $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->limpaMsgCalculoProcesso(
                    $nrcalculomsg, 
                    $nrorg, 
                    $dsorigmsg,
                    $this->hcmParameters->getCdoperador() 
                );

                if ($calculofolha) {
    
                    //Mensagens do processo
                    $idstatus     = MsgCalculo::ID_PENDENTE;
    
                    foreach ($itenslanclote as $itemlanclote) {
                        $processamento = MsgCalculo::getInstance();
                        $processamento->setDados($nrcalculomsg, $itemlanclote->getNrvinculom(), $nrorg, $dsorigmsg);
    
                        $itensliquido = $this->itemLoteLancService->getLiquido($itemlanclote->getNrvinculom(), $calculofolha->getNrcalculofolha(), $nrorg);
    
                        if ($itensliquido) {
                            //$temLiquido = true;
                            $vincLiquido[] = $itemlanclote->getNrvinculom();
                            MsgCalculo::getInstance()->adicionaMensagem('O vinculo ' . $itemlanclote->getNrvinculom() . ' não foi cancelado pois já possui líquido.', MsgCalculo::ERRO);
                            $processamento->salvaMensagens();
                            $processamento->clear();
                        }
                    }
                    
                    if ( $temLiquido ) 
                        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM01020_FPA_EXPORTACAO_LOTE_NAO_CANCELADA, DTO\Response\Message::TYPE_WARNING));
                    
                    else{    
                        
                        /*Se não houver líquido e o lote for oriundo do Lançamento de Frequencia, 
                        é setado null na coluna NRLOTELANCEVENTO da tabela PTO_RESUMOSITUFREQ referente aos vínculos que terão seus ítens excluídos*/
                        if($row['IDORIGLOTELANC'] == 'PTO'){
                            
                            $params = [
                                        'P_NRORG'            => $nrorg,
                                        'P_NRLOTELANCEVENTO' => $row['NRLOTELANCEVENTO']
                                      ];
                            
                            $where = "
                                NRORG = :P_NRORG
                                AND NRLOTELANCEVENTO = :P_NRLOTELANCEVENTO
                            ";
                            
                            $itens_resumositufreq = PtoResumositufreq::getObjectsByWhere($where, $params);

                            foreach ($itens_resumositufreq as $item_resumo){
                                if (array_search($item_resumo->getNrvinculom(), $vincLiquido) === false) {
                                    $item_resumo->setNrlotelancevento(null);
                                    $item_resumo->persist();
                                    \Setforms2\Shared\Registry::getEntityManager()->flush();
                                }
                            }
                            
                            if(count($itens_resumositufreq) > 0) 
                                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM01020_FPA_NR_LOTE_RESUMO_SITU_FREQ_EXLUIDOS, DTO\Response\Message::TYPE_MESSAGE));
                            

                        }
                        
                        /*Se não houver líquido para os vínculos busca os itens de cálculo referentes ao lote e apaga um a um*/
                        $params = [
                                    'P_NRORG'            => $nrorg,
                                    'P_NRCALCULOFOLHA'   => $calculofolha->getNrcalculofolha(),
                                    'P_NRLOTELANCEVENTO' => $row['NRLOTELANCEVENTO']
                                  ];
                        
                        $where = "NRORG = :P_NRORG AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA AND NRLOTELANCEVENTO = :P_NRLOTELANCEVENTO";
                        
                        $itenscalculo = FpaItecalcfolha::getObjectsByWhere($where, $params);
        
                        foreach ($itenscalculo as $itemcalc) {
                            if (array_search($itemcalc->getNrvinculom(), $vincLiquido) === false) {
                                $itemcalc->delete();
                            }
                        }
        
                        /*Atualiza o status do lote de eventos para pendente*/
                        $obj = FpaLotelancevento::getObjectByWhere('NRORG = ? AND NRLOTELANCEVENTO = ?', array($nrorg, $row['NRLOTELANCEVENTO']));
                        $obj->setIdstatuslotelanc('PENDENTE');
                        $obj->persist();
        
                        \Setforms2\Shared\Registry::getEntityManager()->flush();
        
                        $response->addNotification(new DTO\Response\Notification('cancelar'), DTO\Response\Notification::TYPE_SUCCESS);
                    }
                    
                    //Mensagens do processo
                    $msgs = $this->getMensagens($dsorigmsg, $nrcalculomsg);
                    
                    if( count($msgs) > 0 )
                        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('cancelarExportLoteLancEventos', $msgs));

                }else{   
                    
                    if($row['IDORIGLOTELANC'] == 'PTO'){
                            
                        $params = [
                                    'P_NRORG'            => $nrorg,
                                    'P_NRLOTELANCEVENTO' => $row['NRLOTELANCEVENTO']
                                  ];
                        
                        $where = "
                            NRORG = :P_NRORG
                            AND NRLOTELANCEVENTO = :P_NRLOTELANCEVENTO
                        ";
                        
                        $itens_resumositufreq = PtoResumositufreq::getObjectsByWhere($where, $params);

                        foreach ($itens_resumositufreq as $item_resumo){ 
                            $item_resumo->setNrlotelancevento(null);
                            $item_resumo->persist();
                            \Setforms2\Shared\Registry::getEntityManager()->flush();
                        }
                        
                        if(count($itens_resumositufreq) > 0) 
                            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM01020_FPA_NR_LOTE_RESUMO_SITU_FREQ_EXLUIDOS, DTO\Response\Message::TYPE_MESSAGE));
                    }

                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM01020_FPA_NAO_HA_EXPORTACAO_PARA_CANCELAR, DTO\Response\Message::TYPE_WARNING));
                }
            }
                
        } catch (\Exception $e) {
            if ($e->getCode() === Messages::SUCCESS_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_MESSAGE));
            } elseif ($e->getCode() === Messages::ALERT_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_WARNING));
            } elseif ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function exportarLote (DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = $this->dataSourceSetIteCalcFolha;
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                if(array_key_exists('ITEM', $row)){
                    
                    $msgs = array();
                    $dsorigmsg = 'LOTEEVENTO'.$row['NRLOTELANCEVENTO'];
                    $nenhumItemExportado = true;
                    
                    $row['NRCALCULOFOLHA'] = FPA::determinaNrCalculoFolha($row['NRTIPOMOVIMENT']);
                    $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->limpaMsgCalculoProcesso(
                        $row['NRCALCULOFOLHA'], 
                        $this->hcmParameters->getNrorgTrabalho(), 
                        $dsorigmsg,
                        $this->hcmParameters->getCdoperador() 
                    );
                    
                    $row['DTOCORR'] = $row['DTOCORRENCIA'];
                    foreach($row['ITEM'] as $item) {
                        
                        $itemEventoIncide363 = \Setforms2\Shared\Registry::getEntityManager()->getRepository(Repositories::FPA_INCIDEVENTO)->findOneBy(['nrorg' => [$this->hcmParameters->getNrorgPadrao(), $row['NRORG']],
                        'nreventom' => $item['NREVENTOM'], 'nrtipoincide' => 363]);

                        if($itemEventoIncide363) {
                            continue;
                        }

                        $processamento = MsgCalculo::getInstance();
                        $processamento->setDados($row['NRCALCULOFOLHA'], $item['NRVINCULOM'], $this->hcmParameters->getNrorgTrabalho(), $dsorigmsg);
                        $itemLoteDtOcor = $item['DTOCORRENCIA'] ? substr($item['DTOCORRENCIA'],0, 10) : false;
                        
                        if($item['DTOCORRENCIA']){
                            $row['DTOCORRENCIAITEM'] = $item['DTOCORRENCIA'];
                        }else if($row['DTOCORR']){
                           $row['DTOCORRENCIAITEM'] = $row['DTOCORR'];
                        }else{
                            $row['DTOCORRENCIAITEM'] = $row['DTCOMPETLOTEEVE'];
                        }
                        $itenscalcfolha = $this->itemLoteLancService->getLiquido($item['NRVINCULOM'], $row['NRCALCULOFOLHA'], $row['NRORGTRAB']);
                        
                        $demonstra = $this->itemLoteLancService->getIdeventodemonst($row['NRORGPADRAO'], $item['NREVENTOM'], $row['DTMESCOMPETEC'], $row['NRORG']);
                       
                        if(isset($itenscalcfolha[0])){
                            $itenscalcfolha = $itenscalcfolha[0];
                        }
                        
                        if(isset($demonstra[0])){
                            $demonstra = $demonstra[0]["IDEVENTODEMONST"];
                        } 
                        
                        $itemCalc = \Setforms2\Shared\Registry::getEntityManager()->getRepository(Repositories::FPA_ITECALCFOLHA)->findOneBy(array('nrorg' => $row['NRORG'],
                        'nrvinculom' => $item['NRVINCULOM'], 'nrcalculofolha' => $row['NRCALCULOFOLHA'], 'nreventom' => $item['NREVENTOM']));

                        if($itemLoteDtOcor) {
                            $itemCalc = \Setforms2\Shared\Registry::getEntityManager()->getRepository(Repositories::FPA_ITECALCFOLHA)->findOneBy(array('nrorg' => $row['NRORG'],
                            'nrvinculom' => $item['NRVINCULOM'], 'nrcalculofolha' => $row['NRCALCULOFOLHA'], 'nreventom' => $item['NREVENTOM'], 'dtocorrencia' => DateUtil::getDataDeString($itemLoteDtOcor, DateUtil::FORMATO_BRASILEIRO, true)));
                        }
                        
                        if(!empty($item["VREVENTOLOTE"]) && empty($item['VRREFFOLHALOTE'])){
                            $idrefevalorcalc = 'VALOR';
                        }else if (empty($item["VREVENTOLOTE"]) && !empty($item['VRREFFOLHALOTE'])){
                            $idrefevalorcalc = 'REFERENCIA';
                        }else if(!empty($item["VREVENTOLOTE"]) && !empty($item['VRREFFOLHALOTE'])){
                            $idrefevalorcalc = 'REFERENCIA_VALOR';
                        }else{
                            $idrefevalorcalc = 'NORMAL';
                        } 
                        if($this->hcmParameters->getOcorrenciaCalculo() > 1 && !strstr($idrefevalorcalc, '_RECALCULO') && $idrefevalorcalc != 'NORMAL'){
                            $idrefevalorcalc .= '_RECALCULO';
                        }
                        
                        $itemEventoIncide = \Setforms2\Shared\Registry::getEntityManager()->getRepository(Repositories::FPA_INCIDEVENTO)->findOneBy(array('nrorg' => $row['NRORG'],
                        'nreventom' => $item['NREVENTOM'], 'nrtipoincide' => 358));
                        
                        if(empty($itemEventoIncide)){
                            $itemEventoIncide = \Setforms2\Shared\Registry::getEntityManager()->getRepository(Repositories::FPA_INCIDEVENTO)->findOneBy(array('nrorg' => 0,
                            'nreventom' => $item['NREVENTOM'], 'nrtipoincide' => 358));    
                        }
                        
                        if(!empty($itemEventoIncide)){
                            $idrefevalorcalc = 'NORMAL';
                        }
                        
                        if (!$itenscalcfolha || $demonstra == 'S') {
                            $itemCalcDtOcor = $itemCalc ? ($itemCalc->getDtocorrencia() ? $itemCalc->getDtocorrencia()->format('d/m/Y') : false) : false;

                            if(!$itemLoteDtOcor || !$itemCalcDtOcor) { //Se não existe data de ocorrência no item do lote OU não existe esta data no item já existente em FPA_ITECALCFOLHA.
                                if ($itemCalc) { //Edita o item FPA_ITECALCFOLHA existente.
                                    $itemCalc->setVrreffolha($item['VRREFFOLHALOTE']);
                                    $itemCalc->setVreventofolha(round($item["VREVENTOLOTE"],2)); 
                                    $itemCalc->setIdrefevalorcalc($idrefevalorcalc); 
                                    $itemCalc->setDtultatu(DateUtil::getDataAtual());
                                    $itemCalc->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                                    $itemCalc->setCdoperultatu($this->hcmParameters->getCdoperador());
                                    $itemCalc->setNrlotelancevento($item["NRLOTELANCEVENTO"]);
                                    $itemCalc->setDtocorrencia(DateUtil::truncateData(DateUtil::getDataDeString(substr($row['DTOCORRENCIAITEM'],0, 10))));
                                    $itemCalc->setDsitemcalcfolha($item["DSOBSERVACAO"]);
                                    $this->entityManager->persist($itemCalc);
                                    $this->entityManager->flush($itemCalc);
                                }
                                else { //Cria um novo item FPA_ITECALCFOLHA.
                                    $row['DTULTATU'] = $this->hcmParameters->getDataAtual(false);
                                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                                    $row['VRREFFOLHA'] = $item['VRREFFOLHALOTE'];
                                    $row['VREVENTOFOLHA'] = round($item["VREVENTOLOTE"],2);
                                    $row['IDREFEVALORCALC'] = $idrefevalorcalc;
                                    $row['DTINCLUSAO'] = $this->hcmParameters->getDataAtual(false);
                                    $row['CDOPERINCLUSAO'] = $row['CDOPERADOR_LOG'];
                                    $row['NRORGINCLUSAO'] = $row['NRORGTRAB'];
                                    $row['NRITEMCALCFOLHA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORGTRAB'], 'FPA_ITECALCFOLHA', 1, 12, true);
                                    $row['NRORG'] = $row['NRORGTRAB'];
                                    $row['NRVINCULOM'] = $item['NRVINCULOM'];
                                    $row['NREVENTOM'] = $item['NREVENTOM'];
                                    $row['DTOCORRENCIA'] = DateUtil::truncateData(DateUtil::getDataDeString(substr($row['DTOCORRENCIAITEM'],0, 10)))->format('d/m/Y');
                                    $row['DSITEMCALCFOLHA'] = $item['DSOBSERVACAO'];
                                    $row['IDEVENDEMONCALC'] = $demonstra ? $demonstra : 'N';
                                    $row['DTGERULTGRRF'] = null;
                                    
                                    if(!array_key_exists('QTOCORPROXPERI', $row) && !array_key_exists('QTOCORPROXPERI', $item))
                                        $row['QTOCORPROXPERI'] = null;
                                    
                                    if(!array_key_exists('VRVELHO', $row) && !array_key_exists('VRVELHO', $item))
                                        $row['VRVELHO'] = null;
                                    
                                    if(!array_key_exists('DTGERULTCNAB', $row) && !array_key_exists('DTGERULTCNAB', $item))
                                        $row['DTGERULTCNAB'] = null;
                                    
                                    if(!array_key_exists('IDREFEVALORCALC', $row) && !array_key_exists('IDREFEVALORCALC', $item))
                                        $row['IDREFEVALORCALC'] = 'NORMAL';
    
                                    parent::save($request, $response);   
                                }
                            }
                            else if($itemLoteDtOcor && $itemCalcDtOcor) { //Tem a data de ocorrência no item do lote E no item existente em FPA_ITECALCFOLHA.
                                    $datasIguais = DateUtil::comparaDatas($itemLoteDtOcor, $itemCalcDtOcor, 3);

                                    if($datasIguais) { //Sobrepõe os dados do item atual na FPA_ITECALCFOLHA somente qdo a data de ocorrência for igual ao item do lote.
                                        $itemCalc->setVrreffolha($item['VRREFFOLHALOTE']);
                                        $itemCalc->setVreventofolha(round($item["VREVENTOLOTE"],2)); 
                                        $itemCalc->setIdrefevalorcalc($idrefevalorcalc); 
                                        $itemCalc->setDtultatu(DateUtil::getDataAtual());
                                        $itemCalc->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                                        $itemCalc->setCdoperultatu($this->hcmParameters->getCdoperador());
                                        $itemCalc->setNrlotelancevento($item["NRLOTELANCEVENTO"]);
                                        $itemCalc->setDtocorrencia(DateUtil::truncateData(DateUtil::getDataDeString(substr($row['DTOCORRENCIAITEM'],0, 10))));
                                        $itemCalc->setDsitemcalcfolha($item["DSOBSERVACAO"]);
                                        $this->entityManager->persist($itemCalc);
                                        $this->entityManager->flush($itemCalc);
                                    }
                                    else { //Cria um novo item qdo as datas de ocorrência são diferentes.
                                        $row['DTULTATU'] = $this->hcmParameters->getDataAtual(false);
                                        $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                                        $row['NRORGULTATU'] = $row['NRORGTRAB'];
                                        $row['VRREFFOLHA'] = $item['VRREFFOLHALOTE'];
                                        $row['VREVENTOFOLHA'] = round($item["VREVENTOLOTE"],2);
                                        $row['IDREFEVALORCALC'] = $idrefevalorcalc;
                                        $row['DTINCLUSAO'] = $this->hcmParameters->getDataAtual(false);
                                        $row['CDOPERINCLUSAO'] = $row['CDOPERADOR_LOG'];
                                        $row['NRORGINCLUSAO'] = $row['NRORGTRAB'];
                                        $row['NRITEMCALCFOLHA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORGTRAB'], 'FPA_ITECALCFOLHA', 1, 12, true);
                                        $row['NRORG'] = $row['NRORGTRAB'];
                                        $row['NRVINCULOM'] = $item['NRVINCULOM'];
                                        $row['NREVENTOM'] = $item['NREVENTOM'];
                                        $row['DTOCORRENCIA'] = DateUtil::truncateData(DateUtil::getDataDeString(substr($row['DTOCORRENCIAITEM'],0, 10)))->format('d/m/Y');
                                        $row['DSITEMCALCFOLHA'] = $item['DSOBSERVACAO'];
                                        $row['IDEVENDEMONCALC'] = $demonstra ? $demonstra : 'N';
                                        $row['DTGERULTGRRF'] = null;
                                        
                                        if(!array_key_exists('QTOCORPROXPERI', $row) && !array_key_exists('QTOCORPROXPERI', $item))
                                            $row['QTOCORPROXPERI'] = null;
                                        
                                        if(!array_key_exists('VRVELHO', $row) && !array_key_exists('VRVELHO', $item))
                                            $row['VRVELHO'] = null;
                                        
                                        if(!array_key_exists('DTGERULTCNAB', $row) && !array_key_exists('DTGERULTCNAB', $item))
                                            $row['DTGERULTCNAB'] = null;
                                        
                                        if(!array_key_exists('IDREFEVALORCALC', $row) && !array_key_exists('IDREFEVALORCALC', $item))
                                            $row['IDREFEVALORCALC'] = 'NORMAL';
    
                                        parent::save($request, $response);
                                    }
                            }
                            $nenhumItemExportado = false;
                            
                        }else{
                            MsgCalculo::getInstance()->adicionaMensagem('Já existe líquido para o vinculo ' . $item['NRVINCULOM'], MsgCalculo::ERRO);

                            $processamento->salvaMensagens();
                            $processamento->clear();
                        }    
                    }

                    
                    if($nenhumItemExportado)
                        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM01020_FPA_NENHUM_ITEM_EXPORTADO, DTO\Response\Message::TYPE_WARNING));
                    else{
                        $this->itemLoteLancService->updateIdStatusLoteLanc(self::APROVADO, $request->getDataSet()->getRows()[0]['NRLOTELANCEVENTO'], $request->getDataSet()->getRows()[0]['NRORGTRAB']);
                        $response->addNotification(new DTO\Response\Notification('exportar'), DTO\Response\Notification::TYPE_SUCCESS);
                    }
                    //Mensagens do processo
                    $msgs = $this->getMensagens($dsorigmsg, $row['NRCALCULOFOLHA']);
                    if( count($msgs) > 0 )
                        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('exportarLoteLancEventos', $msgs));                    
            
                }else
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM01020_FPA_NAO_HA_ITEM_PARA_EXPORTAR, DTO\Response\Message::TYPE_WARNING));
                
            }
        } catch(\Exception $e) {
            if ($e->getCode() === Messages::SUCCESS_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_MESSAGE));
            } elseif ($e->getCode() === Messages::ALERT_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_WARNING));
            } elseif ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }        
        }
    }

    public function getMensagens($dsorigmsg, $nrcalculofolha){
        $rows = array();
        
        $message = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(
            [
                'nrorg'          => $this->hcmParameters->getNrorgTrabalho(),
                'dsorigmsg'      => $dsorigmsg,
                'nrcalculofolha' => $nrcalculofolha,
            ]
        );
        
        if(count($message) > 0){
            foreach ($message as $mensagem) {
                $rows[] = [
                          "NRVINCULOM"  => $mensagem->getNrvinculom(), 
                          "NRMSGCALC"   => $mensagem->getNrmsgcalc(), 
                          "DSDESCRICAO" => $mensagem->getDescricao(), 
                          "IDTIPOMSG"   => $mensagem->getIdtipomsg() === 'A' ? 'Alerta' : 'Erro',
                          "DTINCLUSAO"  => $mensagem->getDtinclusao()->format('d/m/Y'),
                        ];
            }
        }
        
        return $rows;
        
    }
    
    public function geraItemLote (DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0);
        error_reporting(E_ERROR | E_PARSE);
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataSet = $dataset[0];
            
            if(count($dataSet['LIST_NRESTRUTLEGAL']) < 1 && count($dataSet['LIST_NRESTRUTGEREN']) < 1 && count($dataSet['NRVINCULOM_LIST']) < 1){
                throw new \Exception('Favor informar um filtro para os vinculos a serem contemplados na rotina! (Estura Legal, Estrutura Gerencial ou Vínculos)', 10);
            }
            
            $msg = $this->itemLoteLancService->geraItemLote($dataSet);    
            
            $message = new \Zeedhi\Framework\DTO\Response\Message('Geração de Itens do lote finalizado com sucesso', DTO\Response\Message::TYPE_MESSAGE);
            if($message instanceof \Zeedhi\Framework\DTO\Response\Message){
                $response->addMessage($message);
            }
            if($msg !== null) {
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet("msg", $msg));
            }
        
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }   
    }
    
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}