<?php

namespace HCM\Geral\Controllers;

use HCM\Util\Repositories;
use HCM\Geral\Services\AlteSituacaoFuncionalService;
use HCM\Geral\Services\ESocialService;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use Teknisa\Model\Gestaopessoas\Entities\GpeSitufuncm;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use HCM\Util\DateUtil;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\Util;

class AlteSituacaoFuncionalController extends Crud {

    protected $dataSourceName = 'gpe_altesitufunc';
    protected $dataSourceNameGet = 'getAltesitufunc';
    protected $nmArquivoTemplateCon = 'situacaoFuncional';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC', 'NRSITUFUNCM_LIST', 'P_INCLUIVINCMES');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, AlteSituacaoFuncionalService $alteSituacaoFuncionalService, ESocialService $eSocialService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->alteSituacaoFuncionalService = $alteSituacaoFuncionalService;
        $this->eSocialService = $eSocialService;
        parent::__construct($dataSourceManager);
    }
    
    public function getAtestadoCIDDentroPeriodo(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            foreach ($request->getDataSet()->getRows() as &$row) {
                
                if($row['DTINISITUFUNC']){
                    $cidDentroPeriodo = $this->alteSituacaoFuncionalService->getAtestadoCIDDentroPeriodo($this->hcmParameters->getNrorgTrabalho(), $row['NRVINCULOM'], $row['CDDIAGNOST'], $row['DTINISITUFUNC'], $row['NRALTESITUFUNC']);
                }else{
                    return $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Favor preencher a data de inicio da Situação Funcional.','A'));
                }
                if(!empty($cidDentroPeriodo)){
                    return $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Dentro do período de 60 dias, já existe outro atestado lançado para o mesmo CID. Favor verificar','A'));
                }
            }
        }catch (\Exception $e) {
                if ($e->getCode() != 0) {
                    $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
                } else {
                    $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
                }
        }
    }
    
    public function getAltesitufunc (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $this->removeEmptyConditions($request->getFilterCriteria());
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $filterData = Util::createNewFilterColumnConditionsToCommaImploded($filterData, ['NRSITUFUNCM_LIST']);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function validaExistenciaCnaeXcid (DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                
                $existeCNAExCID = $this->alteSituacaoFuncionalService->retornaExisteCnaeXcid($row['CDDIAGNOST'], $this->hcmParameters->getNrorgTrabalho(), $row['NRVINCULOM'], $this->hcmParameters->getDtCompetencia());
                if($existeCNAExCID){
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(' Atenção! CID relacionado ao NTEP - Nexo Técnico Epidemiológico Previdenciário  do CNAE da estrutura legal. Deseja prosseguir?','A'));
                }
            }
        } catch (\Exception $e) {
            if ($e->getCode() != 0) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }
    
    public function setAltesitufunc (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                
                $liquido = $this->alteSituacaoFuncionalService->verificaLiquidoDef($row);
                if($liquido){
                    return $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('O vínculo possui liquido no mes de competencia','E'));
                }
                
                $existeSituFuncMesmaData = $this->alteSituacaoFuncionalService->validaSituacaoFuncionalMesmaData($row['NRORG'], $row['NRVINCULOM'], $row['DTINISITUFUNC'], $row['NRALTESITUFUNC']);
                if($existeSituFuncMesmaData)
                {
                    return $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Já existe uma Situação Funcional com mesma data de início','E'));
                }
                 
                if (!isset($row['VALIDA'])) {
                    $row['VALIDA'] = false;
                }
                
                //verifica se há algum evento S-2230 em andamento ou sem um S-3000 concluído para possibilitar a alteração do dado

                $mensagem = $this->alteSituacaoFuncionalService->verificaPossibilidadeAlteracao($row['NRALTESITUFUNC'], 'editar', $row['NRVINCULOM']);
                
                if ($mensagem === true || $row['VALIDA']) {
                    
                    if (!isset($row['NRMOTIVOAFASTA'])) {
                        $row['NRMOTIVOAFASTA'] = '';
                    }
                    if (!isset($row['NRPROXSITUFUNC'])) {
                        $row['NRPROXSITUFUNC'] = '';
                    }
                    if (!isset($row['CDDIAGNOST'])) {
                        $row['CDDIAGNOST'] = '';
                    }
                    if (!isset($row['NRPESSOA'])) {
                        $row['NRPESSOA'] = '';
                    }
                    if (!isset($row['CDTABECDI'])) {
                        $row['CDTABECDI'] = '';
                    }

                    $nrorg = $row['NRORG'];
                    $nrsitufuncm = $row['NRSITUFUNCM'];
                    $nrvinculom = $row['NRVINCULOM'];
                    $dtinisitufunc = $row['DTINISITUFUNC'];
                    $dtfimsitufunc = $row['DTFIMSITUFUNC'];
                    $nrqtdiasitufunc = $row['NRQTDIASITUFUNC'];
                    $nrproxsitufunc = $row['NRPROXSITUFUNC'];
                    $cdtabecdi = $row['CDTABECDI'];
                    $dtestabsitufunc = $row['DTESTABSITUFUNC'];
                    $cddiagnost = $row['CDDIAGNOST'];
                    $row['NRGRUPOESPECIE'] = $row['NRGRUPOESPECIE'] ?? null;
                    $row['NRESPECIEBENEFICIO'] = $row['NRESPECIEBENEFICIO'] ?? null;
                    
                    if (!$row['__is_new']) $nraltesitufunc = $row['NRALTESITUFUNC'];
                    else $nraltesitufunc = null;

                    // Bloqueio situação
                        $isAfastamento = $this->alteSituacaoFuncionalService->isAfastamento($nrsitufuncm);
                   
                    
                    if ($isAfastamento) {
                        $vinculoRescindido = $this->alteSituacaoFuncionalService->getRescisaoVinculo($nrvinculom);
                        
                        if ($vinculoRescindido) {
                            return $response->addMessage(new DTO\Response\Message('O vínculo está rescindido!', DTO\Response\Message::TYPE_WARNING));
                        }
                    }
                    // End
                    
                    if ($this->alteSituacaoFuncionalService->retornaProximaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtinisitufunc, $nrproxsitufunc, $nraltesitufunc) === true) {
                        // Valida se já existe situação funcional para a mesma data e dias de afastamento para mesmo CID
                        if ($row['__is_new']) {
                            $this->alteSituacaoFuncionalService->validaExisteDataSituFunc($nrorg, $nrvinculom, $dtinisitufunc);
                        }
        
                        // Valida a quantidade de dias de afastamento para mesmo CID
                        $retorno = $this->alteSituacaoFuncionalService->verificAfastMesmoCid(
                            $nrorg, $nrsitufuncm, $nrvinculom, $dtinisitufunc, $dtfimsitufunc, $nrqtdiasitufunc, $cdtabecdi
                        );

                        if (is_array($retorno)) {
                            if ($nrsitufuncm == GpeSitufuncm::SITUACAO_FUNCIONAL_ATESTADO) {
                                $row['NRQTDIASITUFUNC'] = $retorno['NRQTDIASITUFUNC'];
                                $row['DTFIMSITUFUNC'] = $retorno['DTFIMSITUFUNC'];
                                $response->addMessage(new DTO\Response\Message($retorno['WARNING'], DTO\Response\Message::TYPE_WARNING));
                            }
                            else if ($nrsitufuncm == GpeSitufuncm::SITUACAO_FUNCIONAL_DOENCA_MENOR_15) {
                                $row['NRSITUFUNCM'] = $retorno['NRSITUFUNCM'];
                                $response->addMessage(new DTO\Response\Message($retorno['WARNING'], DTO\Response\Message::TYPE_WARNING));
                            }
                        }

                        $dataCompetencia = DateUtil::getDataDeString(substr(
                            $this->hcmParameters->getDtCompetencia(),
                            0,
                            10),
                            DateUtil::FORMATO_BRASILEIRO);
                        $dataInicioSituacaoFuncional = DateUtil::setDay($row['DTINISITUFUNC'], 1);
                        if ($dataInicioSituacaoFuncional > $dataCompetencia) {
                            $competencCadastro = DateUtil::setDay($row['DTINISITUFUNC'], 1);
                        }
                        if ($dataInicioSituacaoFuncional <= $dataCompetencia) {
                            $competencCadastro = $dataCompetencia;
                        }

                        $this->alteSituacaoFuncionalService->beforeSave($nrorg, $nrvinculom, $competencCadastro, $dtinisitufunc);
                            
                        if (array_key_exists('NRALTESITUFUNC', $row)) {
                            if ($row['NRALTESITUFUNC'] === '') {
                                $row['NRALTESITUFUNC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTESITUFUNC', 1, 12, true);
                            }
                        } else {
                            $row['NRALTESITUFUNC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTESITUFUNC', 1, 12, true);
                        }
                        
                        parent::save($request, $response);
                        // atualiza o gozo de férias e as férias do vínculo caso seja necessário
                        if (isset($row['NRGOZOFERIAS'])) {
                            $this->alteSituacaoFuncionalService->atualizaGozoFerias($nrorg, $row['NRVINCULOM'], $row['NRFERIAS'], $row['NRGOZOFERIAS'], $row['DTFIMGOZO'], $row['SALDO']);
                        }
                        
                        // procura por saldo de gozo de férias para alertar ao usuário de que o vínculo precisa gozar o resto das férias
                        if ($row['NRSITUFUNCM'] == 1) {
                            $message = $this->alteSituacaoFuncionalService->verificaSaldoFerias($nrorg, $row['NRVINCULOM'], $row['DTINISITUFUNC']);
                            
                            if (!empty($message))     
                                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($message, DTO\Response\Message::TYPE_WARNING));
                        }
                        
                        $this->alteSituacaoFuncionalService->atualizaSitufuncHistoricoVinculo($nrorg, $nrvinculom, $nrsitufuncm, $competencCadastro);
                        
                        $retorno = $this->alteSituacaoFuncionalService->geraEstabilidadeSituFunc($nrorg, $nrvinculom, $nrsitufuncm, $dtinisitufunc, $nrqtdiasitufunc, $dtfimsitufunc, $dtestabsitufunc);
                        if (is_string($retorno)) {
                            $response->addNotification(new DTO\Response\Notification($retorno, DTO\Response\Notification::TYPE_SUCCESS));
                        }
                        
                        $this->alteSituacaoFuncionalService->verifDtUltSitufunc($nrorg, $nrsitufuncm, $nrvinculom, $dtinisitufunc, $nrproxsitufunc);
                        
                        if ($nrproxsitufunc) {
                            $proxsituacao = $this->alteSituacaoFuncionalService->verifProxSitufunc($nrorg, $nrvinculom, $dtinisitufunc, $dtfimsitufunc, $nrproxsitufunc, $cdtabecdi, $cddiagnost);
                            if ($proxsituacao && $dtfimsitufunc <= $this->hcmParameters->getUltimoDiaMes()) {
                                $this->alteSituacaoFuncionalService->atualizaSitufuncHistoricoVinculo($nrorg, $nrvinculom, $nrproxsitufunc, $competencCadastro);
                                $response->addNotification(new DTO\Response\Notification(Messages::HCM00000_GPE_ALTESITUFUNC_PROXSITUFUNC_CAD_AUTO_SUCCESS, DTO\Response\Notification::TYPE_SUCCESS));
                            }
                        }
                        
                        if($row['DTFIMSITUFUNC'] && !$nrproxsitufunc){
                            $ret = $this->alteSituacaoFuncionalService->verificaPerdaPeriodoFerias($nrorg, $row);
                            
                            if(!empty($ret)) {
                                $response->addNotification(new \Zeedhi\Framework\DTO\Response\Notification($ret, DTO\Response\Notification::TYPE_ALERT));
                            }
                        }
                        
                        if(array_key_exists('IDENVIAESOCIAL', $row) && $row['IDENVIAESOCIAL'] == 'S'){
                            $row['NRCHAVE'] = $row['NRALTESITUFUNC'];
                            $this->eSocialService->envioEsocialDescentralizado($row, $response);
                        }
                    } else {
                        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Operação bloqueada! <br>Já existe uma alteração funcional com data posterior a alteração funcional adicionada!', DTO\Response\Message::TYPE_WARNING));
                    }
    
                } else {
                    if($mensagem instanceof \Zeedhi\Framework\DTO\Response\Message) {
                        $response->addMessage($mensagem);
                        return $response;
                    }
                }
            }
        } catch (\Exception $e) {
            if ($e->getCode() != 0) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function deleteAltesitufunc(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            
            //verifica se há algum evento S-2230 em andamento ou sem um S-3000 concluído para possibilitar a alteração do dado
            $mensagem = $this->alteSituacaoFuncionalService->verificaPossibilidadeAlteracao($row['NRALTESITUFUNC'], 'excluir', $row['NRVINCULOM']);
            
            
            if ($mensagem === true) {
                $nrorg = $row['NRORG'];
                $nrvinculom = $row['NRVINCULOM'];
                $dtinisitufunc = $row['DTINISITUFUNC'];
                parent::delete($request, $response);
                $this->alteSituacaoFuncionalService->afterDelete($nrorg, $nrvinculom, $dtinisitufunc);
            } else {
                if($mensagem instanceof \Zeedhi\Framework\DTO\Response\Message) {
                    $response->addMessage($mensagem);
                }
            }
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function cadExclAutoAlteSitufunc(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $action = $request->getRoutePath();
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            $vincPossuiGozoArr = array();
            $newVincArr = array();
            $msg = array();
            foreach ($request->getDataSet()->getRows() as &$row) {
                $dtFim = $row['DTFIMSITUFUNC'] ?? $this->hcmParameters->getUltimoDiaMes();
                
                for ( $i = 0; $i < count($row['NRVINCULOM']); $i++ ) {
                    $gozoFerias = $this
                            ->eSocialService
                            ->entityManager
                            ->getRepository(Repositories::FPA_GOZOFERIAS)
                            ->retornaGozosFeriasEntreDatas($nrorg, $row['NRVINCULOM'][$i], DateUtil::getDatadeString($row['DTINISITUFUNC']), DateUtil::getDatadeString($dtFim))
                    ;
                    if ($gozoFerias) {
                        $msg[] = array('msg' => "O vínculo ".$row['NRVINCULOM'][$i]." possui período de gozo de férias coincidente ao lançamento desta situação funcional de afastamento. Favor verificar.");
                        $vincPossuiGozoArr[] = $row['NRVINCULOM'][$i];
                    } else {
                        $newVincArr[] = $row['NRVINCULOM'][$i];
                    }
                }
                $row['NRVINCULOM'] = $newVincArr;

                $nrestrutlegal = $row['NRESTRUTLEGAL'];
                $nrestrutgerenlist = isset($row['NRESTRUTGEREN']) ? $row['NRESTRUTGEREN'] : array();
                $nrvinculomlist = isset($row['NRVINCULOM']) ? $row['NRVINCULOM'] : array();
                $dtinisitufunc = $row['DTINISITUFUNC'];
                $dtfimsitufunc = isset($row['DTFIMSITUFUNC']) ? $row['DTFIMSITUFUNC'] : '';
                $nrsitufuncm = $row['NRSITUFUNCM'];
                $nrproxsitufunc = isset($row['NRPROXSITUFUNC']) ? $row['NRPROXSITUFUNC'] : null;
                $motivoAfastamento = isset($row['NRMOTIVOAFASTA']) ? $row['NRMOTIVOAFASTA'] : null;
                $qntDias = isset($row['NRQTDIASITUFUNC']) ? $row['NRQTDIASITUFUNC'] : null;
                $dsObs = isset($row['DSOBSALTSITFUNC']) ? $row['DSOBSALTSITFUNC'] : null;

                if ($action == '/cadastroAutoAlteSitufunc') {
                    $status = $this->alteSituacaoFuncionalService->cadastroAutomatico($response, $nrorg, $dtmescompetenc, $nrestrutlegal, $nrestrutgerenlist, $nrvinculomlist, $dtinisitufunc, $dtfimsitufunc, $nrsitufuncm, $nrproxsitufunc, $motivoAfastamento, $qntDias, $dsObs);
                }
                if ($action == '/exclusaoAutoAlteSitufunc') {
                    $status = $this->alteSituacaoFuncionalService->exclusaoAutomatica($response, $nrorg, $dtmescompetenc, $nrestrutlegal, $nrestrutgerenlist, $nrvinculomlist, $dtinisitufunc, $dtfimsitufunc, $nrsitufuncm, $motivoAfastamento);
                }
            }
            if ( $vincPossuiGozoArr ) {
                $response->addNotification(new DTO\Response\Notification('','WARNING'));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet("msg", $msg));
            } else {
                $response->addNotification(new DTO\Response\Notification('',$status));
            }
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
    
    /*remove campos de filtros vazios*/
    protected function removeEmptyConditions(FilterCriteria $filterCriteria) {
        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            if ($condition['value'] != null && $condition['value'] != "")
                $conditions[] = $condition;
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );

        return $newFilterCriteria;
    }

    public function baixarPlanilhaAlteSitFunc(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $file = $this->alteSituacaoFuncionalService->baixarPlanilhaAlteSitFunc(
                $this->hcmParameters->getNrorgTrabalho(),
                $this->nmArquivoTemplateCon);

            if($file != null){
                $retorno = ['NMARQUIVO' => $file];
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $retorno));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function importarPlanilhaAlteSitFunc(DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0); 
        
        try{
            $row = $request->getDataSet()->getRows()[0];
            
            $messages = $this->alteSituacaoFuncionalService->importarPlanilhaAlteSitFunc($row);
            if(empty($messages)){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida com sucesso.", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            } else {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida. Existem erros na importação", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msg', $messages));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}