<?php
namespace HCM\Geral\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Geral\Repository\DateTime;
use HCM\Util\DateUtil;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use HCM\Util\Util;

class RateioLancamentoRepository {

    protected  $connection;
    
    public function __construct(Connection $connection, EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function buscaRateioExistente($nrorg, $inicioVigencia, $nrestruturam, $nreventom) {
        $sql = $this->entityManager->getConnection()->prepare(
            "SELECT RAT.* 
            FROM FPA_RATEIO RAT
            WHERE (RAT.NRORG = :P_NRORG 
            AND RAT.DTINIVIGENCIA = :P_DTINIVIGENCIA
            AND RAT.NRESTRUTURAM = :P_NRESTRUTURAM
            AND RAT.NREVENTOM = :P_NREVENTOM
            )"
        );
        
        $sql->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_DTINIVIGENCIA' => $inicioVigencia,
                'P_NRESTRUTURAM' => $nrestruturam,
                'P_NREVENTOM' => $nreventom
            )
        );
        
        return $sql->fetch();
    }

    public function buscaRateioExistenteOcupacao($nrorg, $nrestruturam, $nrocupacaom, $nrrateio) {
        $sql = $this->entityManager->getConnection()->prepare(
            "SELECT RAT.*
            FROM FPA_RATEIOOCUP RAT 
            WHERE (RAT.NRORG = :P_NRORG
            AND RAT.NRRATEIO = :P_NRRATEIO
            AND  RAT.NRESTRUTURAM = :P_NRESTRUTURAM
            AND RAT.NROCUPACAOM = :P_NROCUPACAOM)"
        );
        $sql->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRESTRUTURAM' => $nrestruturam,
                'P_NROCUPACAOM' => $nrocupacaom,
                'P_NRRATEIO' => $nrrateio
            )
        );
        
        return $sql->fetch();
    }

    public function buscaCalculoRateioExistente($nrorg, $nrrateio, $nrvinculom, $dtdialanc) {
        $sql = $this->entityManager->getConnection()->prepare(
            "SELECT RAT.*
            FROM FPA_RATEIOVAL RAT 
            WHERE (RAT.NRORG = :P_NRORG
            AND RAT.NRRATEIO = :P_NRRATEIO
            AND  RAT.NRVINCULOM = :P_NRVINCULOM
            AND RAT.DTDIALANC = :P_DTDIALANC)"
        );
        $sql->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_DTDIALANC' => $dtdialanc,
                'P_NRVINCULOM' => $nrvinculom,
                'P_NRRATEIO' => $nrrateio
            )
        );
        
        return $sql->fetch();
    }

    public function buscaRateioExistenteEstrutura($nrorg, $nrestruturam, $nrrateio) {
        $sql = $this->entityManager->getConnection()->prepare(
            "SELECT RAT.*
            FROM FPA_RATEIOESTRUT RAT 
            WHERE (RAT.NRORG = :P_NRORG
            AND RAT.NRRATEIO = :P_NRRATEIO
            AND    RAT.NRESTRUTURAM = :P_NRESTRUTURAM)"
        );
        $sql->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRESTRUTURAM' => $nrestruturam,
                'P_NRRATEIO' => $nrrateio
            )
        );
        
        return $sql->fetch();
    }

    public function buscaRateioExistenteLancamento($nrorg, $nrrateio, $dtlancamento) {
        $sql = $this->entityManager->getConnection()->prepare(
        "SELECT * FROM FPA_RATEIOLANC RAT WHERE (NRORG = :P_NRORG AND RAT.NRRATEIO = :P_NRRATEIO AND RAT.DTLANCAMENTO = :P_DTLANCAMENTO)"
        );
        
        $sql->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRRATEIO' => $nrrateio,
                'P_DTLANCAMENTO' => $dtlancamento
            )
        );
        
        return $sql->fetch();
    }
    
    public function selectExisteItensRateio($nrorg, $nrrateio, $tabela) {
        $sql = $this->entityManager->getConnection()->prepare(
            "SELECT * FROM $tabela WHERE (NRORG = :P_NRORG AND NRRATEIO = :P_NRRATEIO)"
            );
            
            $sql->execute(
                array(
                    'P_NRORG' => $nrorg,
                    'P_NRRATEIO' => $nrrateio
                )
            );
          
            return $sql->fetch();
    }
    
    public function selectPorcentagemEstrutura($nrorg, $nrrateio)
    {   
        $sql = $this->entityManager->getConnection()->prepare(
            "SELECT RAT.NRRATEIOESTRUT, ( RAT.NRPESO / SUM(RAT.NRPESO) OVER(PARTITION BY RAT.NRORG, RAT.NRRATEIO)) * 100 PERCENT FROM FPA_RATEIOESTRUT RAT WHERE RAT.NRORG = :P_NRORG AND RAT.NRRATEIO = :P_NRRATEIO"
            );
            
            $sql->execute(
                array(
                    'P_NRORG' => $nrorg,
                    'P_NRRATEIO' => $nrrateio
                )
            );
            
            return $sql->fetchAll();
    }
    
    public function deleteValoresGeral($nrorg, $nrrateio){
        $dtiniciomes       = $this->hcmParameters->getPrimeiroDiaDoMes();
        $dtfimmes          = $this->hcmParameters->getUltimoDiaMes();
        
        $sql = $this->entityManager->getConnection()->prepare(
            "DELETE FROM FPA_RATEIOVAL WHERE NRORG = :P_NRORG AND NRRATEIO = :P_NRRATEIO AND DTDIALANC BETWEEN :P_DTINIMES AND :P_DTFIMMES");
            
            $sql->execute(
                array(
                    'P_NRORG' => $nrorg,
                    'P_NRRATEIO' => $nrrateio,
                    'P_DTINIMES' => $dtiniciomes,
                    'P_DTFIMMES' => $dtfimmes
                )
            );
    }

    public function selectPorcentagemOcupacao($nrorg, $nrrateio)
    {
        $sql = $this->entityManager->getConnection()->prepare(
            "SELECT RAT.NRRATEIOOCUP, ( RAT.NRPESO / SUM(RAT.NRPESO) OVER(PARTITION BY RAT.NRORG, RAT.NRRATEIO, RAT.NRESTRUTURAM)) * 100 PERCENT FROM FPA_RATEIOOCUP RAT WHERE RAT.NRORG = :P_NRORG AND RAT.NRRATEIO = :P_NRRATEIO"
            );
            
            $sql->execute(
                array(
                    'P_NRORG' => $nrorg,
                    'P_NRRATEIO' => $nrrateio
                )
            );
            
            return $sql->fetchAll();
    }

    public function setPorcentagemEstrutura($nrorg, $nrrateio, $nrrateioestrut, $vrporcentagem)
    {
        $sql = $this->entityManager->getConnection()->prepare("
            UPDATE FPA_RATEIOESTRUT 
            SET VRPORCENTAGEM     = :P_VRPORCENTAGEM 
            WHERE NRRATEIO     = :P_NRRATEIO
                AND NRRATEIOESTRUT  = :P_NRRATEIOESTRUT
                AND NRORG           = :P_NRORG");

            $sql->execute(
                array(
                    "P_VRPORCENTAGEM"  => $vrporcentagem,
                    "P_NRRATEIO"     => $nrrateio,
                    "P_NRRATEIOESTRUT"  => $nrrateioestrut,
                    "P_NRORG"           => $nrorg
                )
            );
    }

    public function setPorcentagemOcupacao($nrorg, $nrrateio, $nrrateioocup, $vrporcentagem)
    {
        $sql = $this->entityManager->getConnection()->prepare("
            UPDATE FPA_RATEIOOCUP 
            SET VRPORCENTAGEM     = :P_VRPORCENTAGEM 
            WHERE NRRATEIO     = :P_NRRATEIO
                AND NRRATEIOOCUP  = :P_NRRATEIOOCUP
                AND NRORG           = :P_NRORG");

            $sql->execute(
                array(
                    "P_VRPORCENTAGEM"  => $vrporcentagem,
                    "P_NRRATEIO"     => $nrrateio,
                    "P_NRRATEIOOCUP"  => $nrrateioocup,
                    "P_NRORG"           => $nrorg
                )
            );
    }
    
    public function getSituFuncEntrePeriodos($nrorg,$nrvinculom,$nrsitufunc,$dtmescompetenc) {
        $sql = $this->entityManager->getConnection()->prepare("
        SELECT * FROM GPE_ALTESITUFUNC
            WHERE NRORG = :P_NRORG 
            AND NRVINCULOM = :P_NRVINCULOM
            AND NRSITUFUNCM IN ($nrsitufunc)
            AND DTINISITUFUNC >= :P_DTMESCOMPETENC 
            AND DTFIMSITUFUNC <= LAST_DAY(:P_DTMESCOMPETENC)
            ");
        
        $sql->execute(['P_NRORG'        => $nrorg,
                      'P_NRVINCULOM' => $nrvinculom,
                      'P_DTMESCOMPETENC' => $dtmescompetenc]);

        $result = $sql->fetch();
        return $result;
    }

    public function getLancamentos($nrorg, $nrrateio, $dtinicio, $dtfim)
    {   
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        $sql = $this->entityManager->getConnection()->prepare("
        SELECT
    lanc.dtlancamento,
    SUM(vrfinal) vrfinal
FROM
    fpa_rateiolanc lanc
WHERE
        nrrateio = :p_nrrateio
    AND lanc.nrorg = :p_nrorg
    AND ( trunc(lanc.dtlancamento, 'MM') = :p_dtmescompetenc )
    AND ( lanc.dtlancamento >= :p_dtinicio
          OR :p_dtinicio IS NULL )
    AND ( lanc.dtlancamento <= :p_dtfim
          OR :p_dtfim IS NULL )
        GROUP BY LANC.DTLANCAMENTO
        ORDER BY
    lanc.dtlancamento ASC");
        $sql->execute(
            array(
                "P_NRRATEIO"         => $nrrateio,
                "P_NRORG"            => $nrorg,
                "P_DTINICIO"         => $dtinicio,
                "P_DTFIM"            => $dtfim,
                "P_DTMESCOMPETENC"   => $dtmescompetenc
                
            )
        );

        return $sql->fetchAll();
    }

    public function getVinculosEstrutLegal($nrorg, $dtmescompetenc, $nrestrutlegal, $nrestrutgeren = null, $nrsitufuncm = null, $nrtipovinculom = null, $nrvinculoempreg = null)
    {
        $gpevinculoh =  'Teknisa\Model\Gestaopessoas\Entities\GpeVinculoh';
        $gpevinculom =  'Teknisa\Model\Gestaopessoas\Entities\GpeVinculom';
        
        $dtultdiacompetenc = DateUtil::getUltimoDiaDoMes(DateUtil::getDataDeString($dtmescompetenc));
        
        if(is_array($nrestrutgeren)) {
            $condNrestrutgeren = "AND vh.nrestrutgeren IN (" . implode(",",$nrestrutgeren) . ")";
        } else {
            $condNrestrutgeren = null;
        }
        
        
        $sql = <<<SQL
SELECT vm, vh
  FROM $gpevinculom vm
  JOIN $gpevinculoh vh WITH vh.nrvinculom = vm.nrvinculom
                        AND vh.nrorg = vm.nrorg
                        AND vh.dtmescompetenc = (SELECT MAX(vhh.dtmescompetenc)
                                                   FROM $gpevinculoh vhh
                                                  WHERE vhh.dtmescompetenc <= :p_dtmescompetenc
                                                    AND vhh.nrorg = vh.nrorg
                                                    AND vhh.nrvinculom = vh.nrvinculom)
 WHERE vm.nrorg = :p_nrorg
   AND vm.dtadmissaovinc <= :p_ultdiacompetenc
   AND (vm.dtrescisaovinc >= :p_dtmescompetenc OR vm.dtrescisaovinc IS NULL)
   AND (vh.nrestrutlegal = :p_nrestrutlegal or :p_nrestrutlegal IS NULL)
   $condNrestrutgeren
SQL;
        $params = array(
            'p_nrorg'           => $nrorg,
            'p_dtmescompetenc'  => $dtmescompetenc,
            'p_ultdiacompetenc' => $dtultdiacompetenc,
            'p_nrestrutlegal'   => $nrestrutlegal,
        );
        
        $resultado = $this->entityManager->createQuery($sql)->execute($params);

        $vinculo = array();
        $vinculom = Repositories::GPE_VINCULOM;
        foreach ($resultado as $key => $vinc) {
            if($vinc instanceof $vinculom){
                $vinc->historico = $resultado[$key+1];
                $vinculo[] = $vinc;
            }    
        }
        return $vinculo;
    }

    public function getQntdVinculosPorEstrutGerenCargo($nrorg, $nrestrutlegal , $nrsitufuncm = null, $nrtipovinculom = null, $nrvinculoempreg = null)
    {    
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        
        if(!empty($nrsitufuncm)){
            $condNrsitufuncm = " AND VH.NRSITUFUNCM NOT IN ($nrsitufuncm)";
        } else{
            $condNrsitufuncm = "";
        }
        
        if(!empty($nrtipovinculom)){
            $condNrtipovinculom = " AND VM.NRTIPOVINCULOM IN ($nrtipovinculom)";
        } else{
            $condNrtipovinculom = "";
        }
        
        if(!empty($nrvinculoempreg)){
            $condNrvinculoempreg = " AND VM.NRVINCULOEMPREG IN ($nrvinculoempreg)";
        } else{
            $condNrvinculoempreg = "";
        }
        
        $sql = $this->entityManager->getConnection()->prepare(
           "SELECT VH.NRESTRUTGEREN, VH.NRCARGO, COUNT(VH.NRVINCULOM) QNTDVINCULOS
              FROM GPE_VINCULOM VM
              JOIN GPE_VINCULOH VH ON VH.NRORG      = VM.NRORG
                                  AND VH.NRVINCULOM = VM.NRVINCULOM
                                  AND VH.DTMESCOMPETENC = (SELECT DTMESCOMPETENC FROM 
                                                              (SELECT VHH.DTMESCOMPETENC
                                                                 FROM GPE_VINCULOH VHH
                                                                WHERE VHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                                  AND VHH.NRORG = VH.NRORG
                                                                  AND VHH.NRVINCULOM = VH.NRVINCULOM
                                                                ORDER BY VHH.DTMESCOMPETENC DESC)
                                                            WHERE ROWNUM = 1)
             WHERE VM.NRORG = :P_NRORG
               AND VH.NRESTRUTLEGAL = :P_NRESTRUTLEGALRAT
               AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
               AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
               $condNrsitufuncm
               $condNrtipovinculom
               $condNrvinculoempreg
             GROUP BY
                VH.NRESTRUTGEREN,
                VH.NRCARGO"
        );
        
        $sql->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRESTRUTLEGALRAT' => $nrestrutlegal,
                'P_DTMESCOMPETENC'   => $dtmescompetenc
            )
        );
            
        return $sql->fetchAll();
    }

    public function getItensValoresRateio($nrorg, $nrrateio, $dtinicio, $dtfim)
    {   
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        $sql = $this->entityManager->getConnection()->prepare("
        SELECT NRVINCULOM, SUM(VRVALOR) VRVALOR
        FROM FPA_RATEIOVAL
        WHERE NRRATEIO = :P_NRRATEIO 
        AND  NRORG = :P_NRORG
        AND (TRUNC(DTDIALANC,'MM') = :P_DTMESCOMPETENC)
        AND (DTDIALANC >= :P_DTINICIO OR :P_DTINICIO IS NULL)
        AND (DTDIALANC <= :P_DTFIM OR :P_DTFIM IS NULL)
        GROUP BY NRVINCULOM");
        $sql->execute(
            array(
                "P_NRRATEIO"         => $nrrateio,
                "P_NRORG"            => $nrorg,
                "P_DTINICIO"         => $dtinicio,
                "P_DTFIM"            => $dtfim,
                "P_DTMESCOMPETENC"   => $dtmescompetenc
                
            )
        );

        return $sql->fetchAll();
    }
}
