<?php
namespace HCM\Geral\Services;

use HCM\Geral\Repository\PreFechamentoFgtsRepository;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;

class PreFechamentoFgtsService {
    
    public function __construct(PreFechamentoFgtsRepository $preFechamentoFgtsRepository, HCMParameters $hcmParameters) {
        $this->preFechamentoFgtsRepository = $preFechamentoFgtsRepository;
        $this->hcmParameters = $hcmParameters;
        $this->conferenciaFgts = new ConferenciaFgts($preFechamentoFgtsRepository);
        $this->informacoesSinteticasFgts = new InformacoesSinteticasFgts($preFechamentoFgtsRepository);
        $this->informacoesSobreBasesCalculoFgts = new InformacoesSobreBasesCalculoFgts($preFechamentoFgtsRepository);
    }
    
    public function getConferenciaFgts($parameters) {
        return $this->conferenciaFgts->getConferenciaFgts($parameters);
    }
    
    public function getInformacoesSinteticasFgts($parameters) {
        return $this->informacoesSinteticasFgts->getInformacoesSinteticasFgts($parameters);
    }
    
    public function getInformacoesSobreBasesCalculoFgts($parameters) {
        return $this->informacoesSobreBasesCalculoFgts->getInformacoesSobreBasesCalculoFgts($parameters);
    }
}

class ConferenciaFgts {
    
    public function __construct(PreFechamentoFgtsRepository $preFechamentoFgtsRepository) {
        $this->preFechamentoFgtsRepository = $preFechamentoFgtsRepository;
    }
    
    public function getConferenciaFgts($parameters) {
        $conferencias = $this->preFechamentoFgtsRepository->getConferenciaFgts($parameters);
        
        for ($i = 0; $i < count($conferencias); $i++) {
            $conferencias[$i]['CONCLUSAO'] = $this->getConclusao($conferencias[$i]);
        }
        
        return $conferencias;
    }
    
    private function getConclusao($conferencia) {
        $diferenca = $this->hasDiferenca($conferencia);
        
        if ($diferenca) {
            return $diferenca;
        };
        
        return 'Sem Diferença';

    }
    
    private function hasDiferenca($conferencia) {
        $tiposDiferencas = array('BASE', 'FGTS', 'ARREDONDAMENTO', 'SEM_FGTS', 'RETORNOESOCIAL');
        $base = $conferencia['BASE_FGTS'];
        $baseEsocial = $conferencia['BASE_FGTS_ESOCIAL'];
        $fgts = $conferencia['DEPOSITO_FGTS'];
        $fgtsEsocial = $conferencia['DEPOSITO_FGTS_ESOCIAL'];
        $diferencaBase = $conferencia['DIFERENCA_BASE'];
        $diferencaFgts  = $conferencia['DIFERENCA_DEPOSITO'];
        $status = $conferencia['IDSTATUS'];
        
        $diferencas = array(
            "BASE" => (($base != $baseEsocial && abs($diferencaBase) > '.05') && $status != 'M') ? 'Diferença na Base' : false,
            
            "FGTS" => ($fgts != $fgtsEsocial && abs($diferencaFgts) > '.05') && $status != 'M' ? 'Diferença no Depósito' : false,
            
            "ARREDONDAMENTO" => (($base != $baseEsocial || $fgts != $fgtsEsocial) && (abs($diferencaBase) <= '.05' || abs($diferencaFgts) <= '.05') && $status != 'M') ? 'Arredondamento' : false,
            
            "SEM_FGTS" => ($fgts == null && $fgtsEsocial == null || $fgts == 0 && $fgtsEsocial == 0) || 
                              ($fgts == 0 && $fgtsEsocial == null || $fgts == null && $fgtsEsocial == 0) && $status != 'M' ? 'Sem FGTS' : false,
            
            "RETORNOESOCIAL" => ($status == 'M') ? 'Sem retorno do eSocial' : false
        );
        
        foreach ($tiposDiferencas as $tipoDiferenca) {
            $hasDiferenca = $diferencas[$tipoDiferenca];
            
            if ($hasDiferenca) {
                return $hasDiferenca;
            }
        }
        
        return false;
   
    }
        
}

class InformacoesSinteticasFgts {
    
    public function __construct(PreFechamentoFgtsRepository $preFechamentoFgtsRepository) {
        $this->preFechamentoFgtsRepository = $preFechamentoFgtsRepository;
    }
    
    public function getInformacoesSinteticasFgts($parameters) {
        return $this->preFechamentoFgtsRepository->getInformacoesSinteticasFgts($parameters);
    }
    
}

class InformacoesSobreBasesCalculoFgts {
    
    public function __construct(PreFechamentoFgtsRepository $preFechamentoFgtsRepository) {
        $this->preFechamentoFgtsRepository = $preFechamentoFgtsRepository;
    }
    
    public function getInformacoesSobreBasesCalculoFgts($parameters) {
        return $this->preFechamentoFgtsRepository->getInformacoesSobreBasesCalculoFgts($parameters);
    }
    
}