<?php
namespace HCM\SST\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\SST\Services\GheHomogenExposicaoService;
use Zeedhi\Framework\DTO\Response\Notification;

class GheHomogenExposicaoController extends Crud {

    protected $dataSourceName      = 'sst_gherisco';
    protected $dataSourceName_get  = 'sst_gherisco_get';
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, GheHomogenExposicaoService $gheHomogenExposicaoService){
        $this->parameterBag          = $parameterBag;
        $this->hcmParameters         = $hcmParameters;
        $this->gheHomogenExposicaoService = $gheHomogenExposicaoService;
        parent::__construct($dataSourceManager);
    }

    public function getGheHomogenExposicao (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->addCondition('P_NRORG_PADRAO','=',$this->hcmParameters->getNrorgPadrao());
        $filterData->setDataSourceName($this->dataSourceName_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setGheHomogenExposicao (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRGHERISCO', $row)) {
                    if ($row['NRGHERISCO'] === '') {
                        $row['NRGHERISCO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_GHERISCO', 1, 12, true);
                    }
                } else {
                    $row['NRGHERISCO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_GHERISCO', 1, 12, true);
                }
            }
            $row["NRSETOR"]        = isset($row["NRSETOR"]) ?        $row["NRSETOR"] :        null;
            $row["NRSITUGERADORA"] = isset($row["NRSITUGERADORA"]) ? $row["NRSITUGERADORA"] : null;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteGheHomogenExposicao(DTO\Request\DataSet $request, DTO\Response $response){
        parent::delete($request, $response);
    }

    public function setCadastroAutomaticoGHE(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                $this->gheHomogenExposicaoService->setCadastroAutomaticoGHE($row);
            }

        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 