<?php

namespace HCM\SST\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use Doctrine\ORM\EntityManager;
use HCM\SST\Services\RegistroAmbientalService;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Request\DataSet;

class RegistroAmbientalController extends Crud {

    protected $parameterBagColumns = array('P_NRORG', 'P_NRESTRUTURAM');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, EntityManager $entityManager, RegistroAmbientalService $registroAmbientalService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager; 
        $this->registroAmbientalService = $registroAmbientalService;
        parent::__construct($dataSourceManager);
    }

    public function getRegistroAmbiental(DTO\Request\Filter $request, DTO\Response $response)
    {   
        $this->dataSourceName = "sst_respregamb_get";
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setRegistroAmbiental(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "sst_respregamb";
            foreach ($request->getDataSet()->getRows() as &$row) {
            
                $row['NRPARCREGAMB'] = $row['NRPARCREGAMB']??null;
                $row['DTINIRESAMB'] = $row['DTINIRESAMB']??null;
                $row['DTFIMRESAMB'] = $row['DTFIMRESAMB']??null;
                $row['NRESTRUTURAM'] = $row['NRESTRUTURAM']??null;
                $row['IDATIVO'] = $row['IDATIVO']?? 'S';
               
                $existeRegistroAmbiental = $this->registroAmbientalService->existeMesmoPeriodoRegistroAmbiental($row);
                if(!empty($existeRegistroAmbiental) && ($existeRegistroAmbiental['NRREGAMB'] != $row['NRREGAMB'] || !isset($row['NRREGAMB'])) ) {
                    $response->addNotification(new Notification("Já existe um Registro Ambiental para o período informado", Notification::TYPE_ALERT));
                    return;
                }
               
                if($row['__is_new'] === true){
                   $row['NRREGAMB'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_RESPREGAMB', 1, 12, true); 
                }

                parent::save($request, $response);
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteRegistroAmbiental(DTO\Request\DataSet $request, DTO\Response $response)
    {   
        try {
            $this->dataSourceName = "sst_respregamb";
            $this->dataSourceName = $this->dataSourceName;
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition)
    {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria)
    {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 