<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 29/06/2016
 * Time: 17:42
 */

namespace HCM\SST\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class RiscoAmbientalController extends Crud {

    protected $dataSourceName = 'gpe_fatorriscamb';
    protected $dataSourceName2 = 'eso_agentenocivo';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        parent::__construct($dataSourceManager);
    }
    
    public function getFatorRiscAmb (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName('gpe_fatorriscamb');
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function getEsoAgenteNocivo (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName('eso_agentenocivo');
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setFatorRiscAmb(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRFATRISCAMB', $row)) {
                    if ($row['NRFATRISCAMB'] === '') {
                        $row['NRFATRISCAMB'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GPE_FATRISCAMB', 1, 12, true);
                    }
                } else {
                    $row['NRFATRISCAMB'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GPE_FATRISCAMB', 1, 12, true);
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteFatorRiscAmb(DTO\Request\DataSet $request, DTO\Response $response){
        parent::delete($request, $response);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
} 